@@ -23,10 +23,6 @@ from oslo_db.sqlalchemy import utils as sa_utils
 import oslo_log.helpers as log_helpers
 from oslo_log import log
 
-from neutron.callbacks import events
-from neutron.callbacks import exceptions as callbacks_exc
-from neutron.callbacks import registry
-from neutron.callbacks import resources
 import neutron.common.exceptions as ml2_exceptions
 from neutron.conf.plugins.ml2 import config  # noqa
 from neutron.db import _resource_extend as resource_extend
@@ -47,6 +43,7 @@ from neutron.db import models_v2
 from neutron.db import portbindings_db
 from neutron.extensions import providernet as provider
 from neutron.objects.qos import policy as policy_object
+import neutron.objects.router as router_object
 from neutron.plugins.ml2 import managers as n_managers
 from neutron_lib.api.definitions import availability_zone as az_def
 from neutron_lib.api.definitions import external_net
@@ -55,11 +52,16 @@ from neutron_lib.api.definitions import portbindings
 from neutron_lib.api.definitions import provider_net
 from neutron_lib.api import validators
 from neutron_lib.api.validators import availability_zone as az_validator
+from neutron_lib.callbacks import events
+from neutron_lib.callbacks import exceptions as callbacks_exc
+from neutron_lib.callbacks import registry
+from neutron_lib.callbacks import resources
 import neutron_lib.callbacks.resources as attributes
 from neutron_lib import constants
 from neutron_lib import exceptions
 from neutron_lib.exceptions import availability_zone as az_exc
 from neutron_lib.plugins import directory
+import neutronclient.client as neutronclient
 import neutronclient.common.exceptions as q_cli_exceptions
 
 from sqlalchemy import sql
@@ -184,6 +186,7 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
         self.extension_manager.initialize()
         self.type_manager.initialize()
         self.helper = helper.NetworkHelper(self)
+        neutronclient.USER_AGENT = t_constants.CENTRAL
         qos_driver.register()
 
     def _setup_rpc(self):
@@ -1969,11 +1972,7 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
         :return: None
         """
         try:
-            with context.session.begin():
-                fip_qry = context.session.query(l3_db.FloatingIP)
-                floating_ips = fip_qry.filter_by(id=_id)
-                for floating_ip in floating_ips:
-                    floating_ip.update(org_data)
+            router_object.FloatingIP.update_objects(context, org_data, id=_id)
         except Exception as e:
             # log the exception and re-raise it
             LOG.exception('Fail to rollback floating ip data, reason: '
