@@ -13,14 +13,17 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from neutron_lib.plugins.ml2 import api
 from oslo_config import cfg
 from oslo_log import log
 
-from neutron.plugins.ml2 import driver_api
 from neutron.plugins.ml2.drivers import type_vxlan
+from neutron_lib import constants as q_lib_constants
 from neutron_lib import exceptions as n_exc
 
 from tricircle.common import constants
+import tricircle.common.context as t_context
+import tricircle.db.api as db_api
 
 LOG = log.getLogger(__name__)
 
@@ -43,14 +46,24 @@ class VxLANTypeDriver(type_vxlan.VxlanTypeDriver):
     def reserve_provider_segment(self, context, segment):
         res = super(VxLANTypeDriver,
                     self).reserve_provider_segment(context, segment)
-        res[driver_api.NETWORK_TYPE] = constants.NT_VxLAN
+        res[api.NETWORK_TYPE] = constants.NT_VxLAN
         return res
 
     def allocate_tenant_segment(self, context):
         res = super(VxLANTypeDriver,
                     self).allocate_tenant_segment(context)
-        res[driver_api.NETWORK_TYPE] = constants.NT_VxLAN
+        res[api.NETWORK_TYPE] = constants.NT_VxLAN
         return res
 
     def get_mtu(self, physical_network=None):
         pass
+
+    def get_endpoint_by_host(self, host):
+        LOG.debug("get_endpoint_by_host() called for host %s", host)
+        host_endpoint = {'ip_address': None}
+        context = t_context.get_db_context()
+        agents = db_api.get_agent_by_host_type(
+            context, host, q_lib_constants.AGENT_TYPE_OVS)
+        if agents:
+            host_endpoint['ip_address'] = agents['tunnel_ip']
+        return host_endpoint
